@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (c) 2012-2013 Raphaël Barrois
+# Copyright (c) The python-semanticversion project
 # This code is distributed under the two-clause BSD License.
 
 import unittest
@@ -14,25 +14,46 @@ class MatchTestCase(unittest.TestCase):
         '!0.1',
         '<=0.1.4a',
         '>0.1.1.1',
-        '~0.1.2-rc23,1',
-    ]
-
-    valid_specs = [
-        '==0.1.0',
-        '<=0.1.1',
-        '<0.1',
-        '>0.1.2-rc1',
-        '>=0.1.2-rc1.3.4',
-        '==0.1.2+build42-12.2012-01-01.12h23',
         '<0.1.2-rc1.3-14.15+build.2012-01-01.11h34',
     ]
 
+    valid_specs = [
+        '*',
+        '==0.1.0',
+        '=0.1.0',
+        '0.1.0',
+        '<=0.1.1',
+        '<0.1',
+        '1',
+        '>0.1.2-rc1',
+        '>=0.1.2-rc1.3.4',
+        '==0.1.2+build42-12.2012-01-01.12h23',
+        '!=0.1.2-rc1.3-14.15+build.2012-01-01.11h34',
+        '^0.1.2',
+        '~0.1.2',
+        '~=0.1.2',
+    ]
+
     matches = {
-        '==0.1.2': [
+        '*': [
+            '0.1.1',
+            '0.1.1+build4.5',
             '0.1.2-rc1',
+            '0.1.2-rc1.3',
             '0.1.2-rc1.3.4',
             '0.1.2+build42-12.2012-01-01.12h23',
             '0.1.2-rc1.3-14.15+build.2012-01-01.11h34',
+            '0.2.0',
+            '1.0.0',
+        ],
+        '==0.1.2': [
+            '0.1.2+build42-12.2012-01-01.12h23',
+        ],
+        '=0.1.2': [
+            '0.1.2+build42-12.2012-01-01.12h23',
+        ],
+        '0.1.2': [
+            '0.1.2+build42-12.2012-01-01.12h23',
         ],
         '<=0.1.2': [
             '0.1.1',
@@ -41,11 +62,19 @@ class MatchTestCase(unittest.TestCase):
             '0.1.2',
             '0.1.2+build4',
         ],
-        '<0.1.2+': [
+        '!=0.1.2+': [
+            '0.1.2+1',
+            '0.1.2-rc1',
+        ],
+        '!=0.1.2-': [
             '0.1.1',
             '0.1.2-rc1',
-            '0.1.2-rc1.3.4',
-            '0.1.2-rc1+build4.5',
+        ],
+        '!=0.1.2+345': [
+            '0.1.1',
+            '0.1.2-rc1+345',
+            '0.1.2+346',
+            '0.2.3+345',
         ],
         '>=0.1.1': [
             '0.1.1',
@@ -60,37 +89,57 @@ class MatchTestCase(unittest.TestCase):
             '0.2.0',
             '1.0.0',
         ],
-        '>0.1.1+': [
-            '0.1.1+b2',
-            '0.1.2-rc1',
-            '1.1.1',
-            '2.0.4',
-        ],
         '<0.1.1-': [
             '0.1.1-alpha',
             '0.1.1-rc4',
             '0.1.0+12.3',
         ],
+        '^0.1.2': [
+            '0.1.2',
+            '0.1.2+build4.5',
+            '0.1.3-rc1.3',
+            '0.1.4',
+        ],
+        '~0.1.2': [
+            '0.1.2',
+            '0.1.2+build4.5',
+            '0.1.3-rc1.3',
+        ],
+        '~=1.4.5': (
+            '1.4.5',
+            '1.4.10-alpha',
+            '1.4.10',
+        ),
+        '~=1.4': [
+            '1.4.0',
+            '1.6.10-alpha',
+            '1.6.10',
+        ],
     }
 
     def test_invalid(self):
         for invalid in self.invalid_specs:
-            self.assertRaises(ValueError, semantic_version.Spec, invalid)
+            with self.subTest(spec=invalid):
+                with self.assertRaises(ValueError, msg="Spec(%r) should be invalid" % invalid):
+                    semantic_version.Spec(invalid)
 
     def test_simple(self):
         for valid in self.valid_specs:
-            version = semantic_version.Spec(valid)
-            self.assertEqual(valid, str(version))
+            with self.subTest(spec=valid):
+                spec = semantic_version.SpecItem(valid)
+                normalized = str(spec)
+                self.assertEqual(spec, semantic_version.SpecItem(normalized))
 
     def test_match(self):
-        for spec_txt, versions in self.matches.items():
-            spec = semantic_version.Spec(spec_txt)
-            self.assertNotEqual(spec, spec_txt)
-            for version_txt in versions:
-                version = semantic_version.Version(version_txt)
-                self.assertTrue(spec.match(version), "%r does not match %r" % (version, spec))
-                self.assertTrue(semantic_version.match(spec_txt, version_txt))
-                self.assertTrue(version in spec, "%r not in %r" % (version, spec))
+        for spec_text, versions in self.matches.items():
+            for version_text in versions:
+                with self.subTest(spec=spec_text, version=version_text):
+                    spec = semantic_version.Spec(spec_text)
+                    self.assertNotEqual(spec, spec_text)
+                    version = semantic_version.Version(version_text)
+                    self.assertIn(version, spec)
+                    self.assertTrue(spec.match(version), "%r does not match %r" % (version, spec))
+                    self.assertTrue(semantic_version.match(spec_text, version_text))
 
     def test_contains(self):
         spec = semantic_version.Spec('<=0.1.1')
@@ -106,17 +155,14 @@ class MatchTestCase(unittest.TestCase):
         strict_spec = semantic_version.Spec('>=0.1.1-')
         lax_spec = semantic_version.Spec('>=0.1.1')
         version = semantic_version.Version('0.1.1-rc1+4.2')
-        self.assertTrue(version in lax_spec, "%r should be in %r" % (version, lax_spec))
+        self.assertFalse(version in lax_spec, "%r should not be in %r" % (version, lax_spec))
         self.assertFalse(version in strict_spec, "%r should not be in %r" % (version, strict_spec))
 
     def test_build_check(self):
-        strict_spec = semantic_version.Spec('<=0.1.1-rc1+')
-        lax_spec = semantic_version.Spec('<=0.1.1-rc1')
+        spec = semantic_version.Spec('<=0.1.1-rc1')
         version = semantic_version.Version('0.1.1-rc1+4.2')
-        self.assertTrue(version in lax_spec, "%r should be in %r" % (version, lax_spec))
-        self.assertFalse(version in strict_spec, "%r should not be in %r" % (version, strict_spec))
+        self.assertTrue(version in spec, "%r should be in %r" % (version, spec))
 
 
 if __name__ == '__main__':  # pragma: no cover
     unittest.main()
-
